import os
import sys
import math

# Shortcut to log function
log = math.log

# Utilities
def reformat_brewer_color(_color):
    # Convert from JavaScript format (palettes with this format can be obtained from http://colorbrewer2.org/)
    _color = _color.replace("rgb", "")
    _color = _color.replace("(", "")
    _color = _color.replace(")", "")
    _converted_color = [float(_x) / 255 for _x in _color.split(",")]
    return tuple(_converted_color)

def select_color(_val):
    # Select one of the preloaded palettes
    assert 0 < _val < 5, "Color palette value must be between 1 and 4."
    assert isinstance(_val, int), "Color palette value must be an integer."
    _colors = []
    if _val == 1:
        _colors = ['rgb(255,255,204)', 'rgb(255,237,160)', 'rgb(254,217,118)', 'rgb(254,178,76)', 'rgb(253,141,60)',
                   'rgb(252,78,42)', 'rgb(227,26,28)', 'rgb(189,0,38)', 'rgb(128,0,38)']
    elif _val == 2:
        _colors = ['rgb(215,48,39)', 'rgb(244,109,67)', 'rgb(253,174,97)', 'rgb(254,224,144)', 'rgb(255,255,191)',
                   'rgb(224,243,248)', 'rgb(171,217,233)', 'rgb(116,173,209)', 'rgb(69,117,180)']
    elif _val == 3:
        _colors = ['rgb(165,0,38)', 'rgb(215,48,39)', 'rgb(244,109,67)', 'rgb(253,174,97)', 'rgb(254,224,144)',
                   'rgb(255,255,191)', 'rgb(224,243,248)', 'rgb(171,217,233)', 'rgb(116,173,209)', 'rgb(69,117,180)',
                   'rgb(49,54,149)']
    elif _val == 4:
        _colors = ['rgb(103,0,31)', 'rgb(178,24,43)', 'rgb(214,96,77)', 'rgb(244,165,130)', 'rgb(253,219,199)',
                   'rgb(247,247,247)', 'rgb(209,229,240)', 'rgb(146,197,222)', 'rgb(67,147,195)', 'rgb(33,102,172)',
                   'rgb(5,48,97)']
    _colors = [reformat_brewer_color(color) for color in _colors]
    return _colors+_colors

def bytes_to_human(_bytes):
    units = ["B", "KB", "MB", "GB"]
    unit = units[int(log(_bytes, 1024))]
    val = (_bytes*1.) / 1024. ** int(log(_bytes, 1024))
    if ((_bytes*1.) % 1024. ** int(log(_bytes, 1024))) == 0:
       val = int(val)
    to_return = str(val)+unit
    return to_return

def test_line(_line):
    try:
        tmp = _line.split()
        if len(tmp) != 4:
            raise ValueError
        int(tmp[0])
        int(tmp[1])
        float(tmp[2])
        float(tmp[3])
        return True
    except ValueError:
        return False


def remove_one_parentheses(text):
    for i, l in enumerate(text):
        if l == "(":
            if text[i-1] in ["a", "b", "c", "d"]:
                open_index = i
                close_index = i+text[i:].find(")")
                text = text[0:open_index]+text[close_index+1:]
                return True, text
    return False, text


def remove_parentheses(text):
    condition = True
    while condition:
        condition, text = remove_one_parentheses(text)
    return text


def identify_independent_variables(text):
    variables = []
    for i, l in enumerate(text):
        if l == "(":
            if text[i-1] in ["a", "b", "c", "d"]:
                open_index = i
                close_index = i+text[i:].find(")")
                variables.append(text[open_index-1:close_index+1].replace(" ", ""))
    return len(set(variables))


def count_flops(t):
    t = remove_parentheses(t)
    return t.count("+")+t.count("*")+t.count("-")+t.count("/")
