from Functions import *
from matplotlib import pyplot as plt
try:
    import prettyplotlib as ppl
except ImportError:
    ppl = plt
# Switch backend to Agg to allow plotting without DISPLAY enabled.(i.e. in jobs)
plt.switch_backend("Agg")

# Select color palette
colors = select_color(2)

# Input File test
if len(sys.argv) != 3:
    print "Inputs must be a data file and FlopsPerByte ratio."
    exit(1)

filename = sys.argv[1]
if not os.path.isfile(filename):
    print filename, "is not a valid input file."
    exit(1)

RatioFlopPerByte = 0.0
try:
    RatioFlopPerByte = float(sys.argv[2])
    print "FlopsPerByte: ", RatioFlopPerByte
except ValueError:
    print "Warning, the FlopsPerByte ratio must be a float"
    RatioFlopsPerByte = 0.0

with open(filename, "r") as f:
    data = [line for line in f if test_line(line)]
    CoresUsed = [int(line.split()[0]) for line in data]
    MemoryUsed = [int(line.split()[1]) for line in data]
    BytePerCycle = [float(line.split()[2]) for line in data]
    TotalBytePerCycle = [x*y for x,y in zip(BytePerCycle, CoresUsed)]
    TotalGBytePerSec = [3.3*x for x in TotalBytePerCycle]

    AnalyticFLOPS = [RatioFlopPerByte * x for x in TotalGBytePerSec]
    case = 1

    AF = [x for x, y in zip(AnalyticFLOPS, CoresUsed) if y == case]
    Mu = [x for x, y in zip(MemoryUsed, CoresUsed) if y == case]
    Peak = [ 3.3 * 8 for _x in Mu]
    PercentageOfPeak = [100*x/y for x, y in zip(AF, Peak)]

    ppl.scatter(Mu, PercentageOfPeak, s=50)

    plt.xscale("log", basex=2)
    plt.xlabel("Memory Used")
    
    plt.xticks(Mu,[bytes_to_human(x) for x in Mu])
    plt.ylabel("% of peak")
    plt.ylim([0, 100 ])
    
    # ppl.legend(loc="best")
    plt.title("Percentage of peak")
    
    # plt.show()
    fig = plt.gcf()
    fig.set_size_inches(12, 8)
    fig.savefig('percentage.png', dpi=100)

