from Functions import * 
from matplotlib import pyplot as plt
try:
    import prettyplotlib as ppl
except:
    ppl = plt
# Switch backend to Agg to plow when the DISPLAY is not set
plt.switch_backend("Agg")
# Input File comprovations
if len(sys.argv) != 3:
    print "Inputs must be a data file and FlopsPerByte ratio."
    exit(1)

filename = sys.argv[1]
if not os.path.isfile(filename):
    print filename,"is not a valid input file."
    exit(1)

try:
   RatioFlopPerByte = float(sys.argv[2])
   print "FlopsPerByte: ", RatioFlopPerByte
except:
   print "Warning, the FlopsPerByte ratio must be a float"
   RatioFlopsPerByte = 0.0

#RatioFlopPerByte = 6./56.
with open(filename, "r") as f:
    data = [line for line in f if test_line(line)]
    CoresUsed = [int(line.split()[0]) for line in data]
    MemoryUsed = [int(line.split()[1]) for line in data]
    BytePerCycle = [float(line.split()[2]) for line in data]
    FlopPerByte = [float(line.split()[3]) for line in data]
    TotalBytePerCycle = [x*y for x,y in zip(BytePerCycle, CoresUsed)]
    TotalGBytePerSec = [3.3*x for x in TotalBytePerCycle]

    AnalyticFLOPS = [RatioFlopPerByte * x for x in TotalGBytePerSec]
    MeasuredFLOPS = [x*y for x,y in zip(FlopPerByte, TotalGBytePerSec)]
    case = 1

    FpB = [x for x, y in zip(FlopPerByte, CoresUsed) if y == case]
    AF = [x for x, y in zip(AnalyticFLOPS, CoresUsed) if y == case]
    MF = [x for x, y in zip(MeasuredFLOPS, CoresUsed) if y == case]
    Mu = [x for x, y in zip(MemoryUsed, CoresUsed) if y == case]
    OverCounting = [x/y for x, y in zip(MF, AF)]
    maxy = max([max(AF), max(MF)])*1.25

#ppl.scatter(MemoryUsed, BytePerCycle, s=50, label=filename, c=CoresUsed)
ppl.scatter(Mu, AF, s=50, label="Real")
ppl.scatter(Mu, MF, s=50 ,color="r",label="Measured PAPI_DP_OPS", marker="^")



# L1 size line
plt.plot([2 ** 15, 2 ** 15], [0, maxy], "-.")
plt.annotate('L1 32KB', xy=(2**13.2, 0.75*maxy), xytext=(2**13.2, .75*maxy))
# L2 size line
plt.plot([2 ** 18, 2 ** 18], [0, maxy], "-.")
plt.annotate('L2 256KB', xy=(2**16., .6*maxy), xytext=(2**16., .7*maxy))
# L3 private size line
plt.plot([2 ** 21, 2 ** 21], [0, maxy], "-.")
plt.annotate('L3 non-shared 2.5MB', xy=(2**19., .5*maxy), xytext=(2**19., .65*maxy))
# L3 size line
plt.plot([2 ** 24, 2 ** 24], [0, maxy], "-.")
plt.annotate('L3 20MB', xy=(2**22.5, .4*maxy), xytext=(2**22.5, .6*maxy))

plt.annotate('Main Memory', xy=(2**24.5, .4*maxy), xytext=(2**24.5, .6*maxy))

plt.xscale("log", basex=2)
plt.yscale("log")
plt.xlabel("Memory used x core")
plt.ylabel("GFLOPs")
plt.ylim([10**0, 10**(log(maxy)+0.3)])
ppl.legend(loc="upper left")

ax2 = plt.twinx()
ax2.plot(Mu, OverCounting, "--", label="Measured/Real")
ax2.set_ylim([0, max(OverCounting)*1.2])
plt.yscale("linear")
ax2.set_ylabel("OverCounting (Measured/Real)")
ax2.plot([2**13.5, 2**26.5], [1, 1], "-.")
ppl.legend(loc="best")
for tl in ax2.get_yticklabels():
    tl.set_color('b')

plt.savefig("plot.png")
