from Functions import *
from matplotlib import pyplot as plt
try:
    import prettyplotlib as ppl
except ImportError:
    ppl = plt

# Switch backend to Agg to allow plotting without DISPLAY enabled.(i.e. in jobs)
plt.switch_backend("Agg")

# Select palette (options 1 to 4)
colors = select_color(2)

# Input File test
if len(sys.argv) != 3:
    print "Inputs must be a data file and FlopsPerByte ratio."
    exit(1)
filename = sys.argv[1]
if not os.path.isfile(filename):
    print filename, "is not a valid input file."
    exit(1)

RatioFlopPerByte = 0.0
try:
    RatioFlopPerByte = float(sys.argv[2])
    print "FlopsPerByte: ", RatioFlopPerByte
except ValueError:
    print "Warning, the FlopsPerByte ratio must be a float"
    RatioFlopsPerByte = 0.0

maxy = 0

# Open the file
with open(filename, "r") as f:
    # Read data from file
    data = [line for line in f if test_line(line)]
    # Parsing info
    CoresUsed = [int(line.split()[0]) for line in data]
    MemoryUsed = [int(line.split()[1]) for line in data]
    # Measured Bytes per cycle that reach the CPU
    BytePerCycle = [float(line.split()[2]) for line in data]
    # Measured Flops per byte using PAPI counters (not reliable)
    FlopPerByte = [float(line.split()[3]) for line in data]
    # Multiplication of the bytes per cycle per core and the number of used cores
    TotalBytePerCycle = [x*y for x, y in zip(BytePerCycle, CoresUsed)]
    # Multiplication of the total number of bytes per cycle and the processor speed
    TotalGBytePerSec = [3.3*x for x in TotalBytePerCycle]
    # Calculation of the number of flops using the number of FlopsPerByte analytically
    # calculed and the measured bandwidth that reach the cores
    AnalyticFLOPS = [RatioFlopPerByte * x for x in TotalGBytePerSec]
    # Measured number of FLOPS multiplying the measured FlopPerByte ratio (non reliable)
    # with the measured bandwidth
    MeasuredFLOPS = [x*y for x, y in zip(FlopPerByte, TotalGBytePerSec)]
    # Here we plot the Cores - GFLOPs for each of the different memory sizes.
    for index, case in enumerate(sorted(set(MemoryUsed))):
        FpB = [x for x, y in zip(FlopPerByte, MemoryUsed) if y == case]
        AF = [x for x, y in zip(AnalyticFLOPS, MemoryUsed) if y == case]
        Cu = [x for x, y in zip(CoresUsed, MemoryUsed) if y == case]
        maxy = max([maxy, max(AF)])
        ppl.plot(Cu, AF, lw=1, label=bytes_to_human(case), color=colors[index])

    # Compute and plot the theoretical peak performance multiplying the turbo frequency (3.3GHz) 
    # for the number of FlopsPerCycle (8 double precision operations) and the number of cores.
    peak = [3.3 * 8 * _x for _x in Cu]
    ppl.plot(Cu, peak, "k--", label="Peak Performance")
    maxy = max([maxy, max(peak)])
    # Configure the plot
    plt.xlabel("Cores")
    plt.ylabel("GFLOPs")
    plt.xscale("log", basex=2)
    plt.yscale("log")
    plt.ylim([0, maxy])
    ppl.legend(loc="upper left")
    plt.title("Scalability Plot")
    fig = plt.gcf()
    fig.set_size_inches(12, 8)
    fig.savefig('scalability.png', dpi=100)

